#!/bin/sh

#
# Copyright (c) 2010 Peter Holm <pho@FreeBSD.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $FreeBSD: projects/stress2/misc/umountf4.sh 197027 2009-09-09 10:13:56Z pho $
#

# Vnode reference leak test scenario.
# Will fail with  "umount: unmount of /mnt5 failed: Device busy"
# vnode leak not seen on HEAD.

[ `id -u ` -ne 0 ] && echo "Must be root!" && exit 1

. ../default.cfg

mount | grep "${mntpoint}" | grep -q md${mdstart}${part} && umount -f $mntpoint
mdconfig -l | grep -q md$mdstart &&  mdconfig -d -u $mdstart

mdconfig -a -t swap -s 12m -u $mdstart
bsdlabel -w md$mdstart auto
newfs -U md${mdstart}${part} > /dev/null
mount /dev/md${mdstart}${part} $mntpoint
chmod 777 ${mntpoint}

export runRUNTIME=5m
export LOAD=100
export swapLOAD=100
export RUNDIR=${mntpoint}/stressX

for i in `jot 10`; do
	su $testuser -c "\
	cd ..; testcases/run/run testcases/mmap/mmap testcases/swap/swap
	"

	sync;sync;sync
	sleep 5
	n=0
	while mount | grep -qw $mntpoint; do
		umount ${mntpoint} > /dev/null 2>&1 || sleep 0.1
		n=$((n + 1))
		if [ $n -gt 25 ]; then
			echo "*** Leak detected ***"
			fstat $mntpoint
			exit 1
		fi
	done
	mount /dev/md${mdstart}${part} $mntpoint
done

umount $mntpoint
mdconfig -d -u $mdstart
